#!/bin/bash
set -e

pushd "$(dirname "$0")" >/dev/null
REL_PATH="../"
pushd "$REL_PATH" >/dev/null
ABS_PATH=$PWD 
popd >/dev/null

export OPENRADIOSS_PATH=$ABS_PATH
export RAD_CFG_PATH=$OPENRADIOSS_PATH/hm_cfg_files
export RAD_H3D_PATH=$OPENRADIOSS_PATH/extlib/h3d/lib/linux64
export LD_LIBRARY_PATH=$OPENRADIOSS_PATH/extlib/h3d/lib/linux64:$OPENRADIOSS_PATH/extlib/hm_reader/linux64
export OMP_STACKSIZE=400m
export PATH=$OPENRADIOSS_PATH/extlib/hm_reader/linux64:$PATH
export PATH=$OPENRADIOSS_PATH/extlib/intelOneAPI_runtime/linux64:$PATH

file="$1"
nt="$2"
np="$3"
sp="$4"
vtk="$5"
csv="$6"
starter="$7"

runningDirectory=""
jobName=""
extension=""

for A in "$file"; do
  runningDirectory=$(dirname "$A")
  jobName=$(basename "$A" | cut -f1 -d.)
  extension=".$(basename "$A" | cut -s -f2- -d.)"
done

if [ -z "$extension" ]; then
  echo "Check input file."
  echo "Error termination."
  exit 1
fi

if [ "$extension" != ".rad" ]; then
  if [ "$extension" != ".k" ] && [ "$extension" != ".key" ]; then
    echo "Check input file."
    echo "Error termination."
    exit 1
  fi
fi

if [ "$extension" == ".rad" ]; then
  jobSuffix="${jobName: -4}"
  jobName="${jobName::-5}"
  echo "$jobSuffix" | grep -E "[0-9]{4}" >/dev/null
  if [ $? -ne 0 ]; then
    echo "Check input file."
    echo "Error termination."
    exit 1
  fi
fi

if [ "$extension" == ".rad" ]; then
  runID="$jobSuffix"
else
  runID=0
fi

starterexec=/exec/starter_linux64_gf
engineexec=/exec/engine_linux64_gf
enginempiexec=/exec/engine_linux64_gf_ompi

if [ "$sp" == "sp" ]; then 
  starterexec=/exec/starter_linux64_gf_sp
  engineexec=/exec/engine_linux64_gf_sp
  enginempiexec=/exec/engine_linux64_gf_ompi_sp
fi

pushd "$runningDirectory" >/dev/null
if [ "$runID" -eq 0 ]; then
  find . -maxdepth 1 -type f -name "${jobName}.h3d" -delete
  find . -maxdepth 1 -type f -name "${jobName}_[0-9][0-9][0-9][0-9].out" -delete
  find . -maxdepth 1 -type f -name "${jobName}_[0-9][0-9][0-9][0-9].ctl" -delete
  find . -maxdepth 1 -type f -name "${jobName}_[0-9][0-9][0-9][0-9]_[0-9][0-9][0-9][0-9].rst" -delete
  find . -maxdepth 1 -type f -name "${jobName}A[0-9][0-9][0-9]" -delete
  find . -maxdepth 1 -type f -name "${jobName}A[0-9][0-9][0-9].vtk" -delete
  find . -maxdepth 1 -type f -name "${jobName}T[0-9][0-9]" -delete
  find . -maxdepth 1 -type f -name "${jobName}T[0-9][0-9].csv" -delete

  if [ "$extension" == ".rad" ]; then
    if [ "$starter" == "no" ]; then
      echo "$jobName"_${jobSuffix} > "./running_st_$jobName"
      jobFileFullPath="$runningDirectory/$jobName"_${jobSuffix}$extension
    else
    echo "$jobName"_${jobSuffix} > "./stopping_st_$jobName"
    jobFileFullPath="$runningDirectory/$jobName"_${jobSuffix}$extension
    fi
  else
    if [ "$starter" == "no" ]; then
      echo "$jobName"${jobSuffix} > "./running_st_$jobName"
      jobFileFullPath="$runningDirectory/$jobName"${jobSuffix}$extension
    else
    echo "$jobName"${jobSuffix} > "./stopping_st_$jobName"
    jobFileFullPath="$runningDirectory/$jobName"${jobSuffix}$extension
    fi
  fi 
  "$OPENRADIOSS_PATH$starterexec" -i "$jobFileFullPath" -nt "$nt" -np "$np"
    find . -maxdepth 1 -type f -name "running_st_${jobName}" -delete

if [ "$extension" == ".rad" ]; then
  find . -maxdepth 1 -type f -name "${jobName}_${jobSuffix}_0001.rst" >/dev/null
  if [ $? -ne 0 ]; then
    exit 1
  fi
else
  find . -maxdepth 1 -type f -name "${jobName}_0000_0001.rst" >/dev/null
  if [ $? -ne 0 ]; then
    exit 1
  fi
fi
  runID=$((runID + 1))
  jobSuffix=$(printf "%04d" "$runID")
  checkforanotherengine="${jobName}_${jobSuffix}.rad"
  if [ ! -f "$checkforanotherengine" ]; then
     echo Starter complete, no engines found
     exit 1
  fi
  
fi

if [ "$starter" == "no" ] && [ "$runID" -ne 0 ] ; then
  while true; do
    echo "$jobName"_${jobSuffix} > "./running_en_$jobName"
    jobFileFullPath="$runningDirectory/$jobName"_${jobSuffix}.rad
    if [ "$np" == "1" ]; then 
        "$OPENRADIOSS_PATH$engineexec" -i "$jobFileFullPath" -nt "$nt"
    else   
        mpiexec -n "$np" "$OPENRADIOSS_PATH$enginempiexec" -i "$jobFileFullPath" -nt "$nt"
    fi
    rm running_en_$jobName
    
    find . -maxdepth 1 -type f -name "$jobName"_${jobSuffix}"_0001.rst" >/dev/null
    if [ $? -ne 0 ]; then
      exit 1
    fi
    
    runID=$((runID + 1))
    jobSuffix=$(printf "%04d" "$runID")

    checkforanotherengine="${jobName}_${jobSuffix}.rad"
    if [ ! -f "$checkforanotherengine" ]; then
       break
    fi
  done
fi

if [ "$vtk" = "yes" ]; then
  if ! ls | grep -qE "$jobName""A[0-9][0-9][0-9]"; then
    echo
    echo
    echo ---------------------------------------------------------------------
    echo NB: Anim-vtk option selected, but not run since no Anim files present
    echo ---------------------------------------------------------------------
  else
    echo
    echo
    echo ------------------------------------------------------
    echo Anim-vtk option selected, Converting Anim Files to vtk
    echo ------------------------------------------------------
    echo

    for file in $(ls | grep -E "$jobName""A[0-9][0-9][0-9]"); do
      echo Anim File Being Converted is $file
      "$OPENRADIOSS_PATH/exec/anim_to_vtk_linux64_gf" "$file" > "$file.vtk"
    done

    echo
    echo ------------------------------------
    echo Anim file conversion to vtk complete
    echo ------------------------------------
  fi
fi

if [ "$csv" = "yes" ]; then
  if ! ls | grep -qE "$jobName""T[0-9][0-9]"; then
    echo
    echo
    echo ----------------------------------------------------------------
    echo NB: TH-csv option selected, but not run since no T files present
    echo ----------------------------------------------------------------
  else
    echo
    echo
    echo --------------------------------------------------
    echo TH-csv option selected, Converting TH Files to csv
    echo --------------------------------------------------
    echo

    for file in $(ls | grep -E "$jobName""T[0-9][0-9]"); do
      echo TH File Being Converted is $file
      "$OPENRADIOSS_PATH/exec/th_to_csv_linux64_gf" "$file"
    done

    echo
    echo ----------------------------------
    echo TH file conversion to csv complete
    echo ----------------------------------
  fi
fi

popd >/dev/null 
